@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 
 ###################
-#    This tool run scripts and display the result in a Web Interface.
-#    Copyright (C) 2021, 2022  Maurice Lambert
+#    This tool runs CLI scripts and displays output in a Web Interface.
+#    Copyright (C) 2021, 2022, 2023  Maurice Lambert
 
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -20,20 +20,19 @@
 ###################
 
 """
-This tool run scripts and display the result in a Web Interface.
+This tool runs CLI scripts and displays output in a Web Interface.
 
 This file is the "main" file of this package (implements the main function,
 the Server class and the Configuration class).
 """
 
-__version__ = "1.0.0"
+__version__ = "1.0.1"
 __author__ = "Maurice Lambert"
 __author_email__ = "mauricelambert434@gmail.com"
 __maintainer__ = "Maurice Lambert"
 __maintainer_email__ = "mauricelambert434@gmail.com"
 __description__ = """
-This tool run scripts and display the result in a Web
-Interface.
+This tool runs CLI scripts and displays output in a Web Interface.
 
 This file is the "main" file of this package (implements the main function,
 the Server class and the Configuration class).
@@ -42,7 +41,7 @@ license = "GPL-3.0 License"
 __url__ = "https://github.com/mauricelambert/WebScripts"
 
 copyright = """
-WebScripts  Copyright (C) 2021, 2022  Maurice Lambert
+WebScripts  Copyright (C) 2021, 2022, 2023  Maurice Lambert
 This program comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions.
@@ -64,8 +63,6 @@ from json.decoder import JSONDecodeError
 from logging.config import fileConfig
 from collections import defaultdict
 from wsgiref import simple_server
-
-# from logging import basicConfig
 from urllib.parse import quote
 from threading import Thread
 from base64 import b64decode
@@ -97,10 +94,6 @@ if __package__:
         get_environ,
         get_file_content,
         get_arguments_count,
-        # doRollover,
-        # rotator,
-        # namer,
-        # Handler,
         get_real_path,
         WebScriptsSecurityError,
         WebScriptsArgumentError,
@@ -138,10 +131,6 @@ else:
         get_environ,
         get_file_content,
         get_arguments_count,
-        # doRollover,
-        # rotator,
-        # namer,
-        # Handler,
         get_real_path,
         WebScriptsSecurityError,
         WebScriptsArgumentError,
@@ -164,9 +153,6 @@ Content = TypeVar(
     "Content", List[Dict[str, JsonValue]], Dict[str, JsonValue], bytes
 )
 
-# current_directory: str = getcwd()
-# log_path: str = join(current_directory, "logs")
-
 
 class Configuration(DefaultNamespace):
 
@@ -181,6 +167,7 @@ class Configuration(DefaultNamespace):
         "urls": {},
         "modules": [],
         "js_path": [],
+        "cgi_path": [],
         "log_level": 0,
         "statics_path": [],
         "scripts_path": [],
@@ -193,6 +180,7 @@ class Configuration(DefaultNamespace):
         "documentations_path": [],
         "accept_unknow_user": True,
         "force_file_permissions": True,
+        "auth_failures_to_blacklist": None,
         "accept_unauthenticated_user": True,
     }
     __required__ = ("interface", "port")
@@ -238,11 +226,9 @@ class Configuration(DefaultNamespace):
         "accept_unknow_user": bool,
         "accept_unauthenticated_user": bool,
         "admin_groups": List[int],
-        # "urls": dict,
         "modules": list,
         "modules_path": list,
         "js_path": list,
-        # "log_level": int,
         "statics_path": list,
         "documentations_path": list,
         "exclude_auth_paths": list,
@@ -262,7 +248,6 @@ class Configuration(DefaultNamespace):
 
     @log_trace
     def add_conf(self, **kwargs):
-
         """
         Add configurations from other configuration files found.
         """
@@ -270,7 +255,6 @@ class Configuration(DefaultNamespace):
         logger_info("Add configurations...")
 
         for key, value in kwargs.items():
-
             if value is not None:
                 dict_ = self.__dict__
 
@@ -278,7 +262,6 @@ class Configuration(DefaultNamespace):
                 default_value = dict_.get(key)
 
                 if isinstance(default_value, list):
-
                     if isinstance(value, str):
                         logger_debug(
                             "Add configuration list using INI/CFG syntax."
@@ -336,7 +319,6 @@ class Server:
         self.port: int = configuration.port
 
         logger_debug("Create default value for WebScripts server...")
-        # self.user: Dict[str, User] = {}
         self.unknow: Dict = {"id": 1, "name": "Unknow", "groups": [0, 1]}
         self.not_authenticated: Dict = {
             "id": 0,
@@ -396,7 +378,6 @@ class Server:
     def set_default_headers(
         headers: Dict[str, str], security: bool, configuration: Configuration
     ) -> None:
-
         """
         This function sets defaults headers.
         """
@@ -457,7 +438,6 @@ class Server:
 
     @log_trace
     def check_blacklist(self, user: User, ip: str) -> bool:
-
         """
         This function checks that the IP and the
         username are not in the blacklist.
@@ -490,7 +470,6 @@ class Server:
 
     @log_trace
     def get_session(self, cookies: List[str], ip: str) -> User:
-
         """
         This function return User from cookies.
         """
@@ -528,7 +507,6 @@ class Server:
     def use_basic_auth(
         credentials: str, pages: Pages, *args
     ) -> Tuple[str, Dict[str, str], str]:
-
         """
         This function decodes basic auth and
         authenticates user with it.
@@ -554,7 +532,6 @@ class Server:
 
     @log_trace
     def check_auth(self, environ: _Environ) -> Tuple[User, bool]:
-
         """
         This function check if user is authenticated and blacklisted.
         """
@@ -644,7 +621,6 @@ class Server:
 
     @log_trace
     def add_module_or_package(self) -> None:
-
         """
         This function add packages and modules to build custom page.
         """
@@ -682,7 +658,6 @@ class Server:
 
     @log_trace
     def add_paths(self) -> None:
-
         """
         This function add js, static and scripts paths.
         """
@@ -711,7 +686,6 @@ class Server:
                     statics_paths,
                 ),
             ):
-
                 for glob in globs:
                     glob = join(dirname_, normpath(glob))
                     logger_debug(
@@ -740,7 +714,6 @@ class Server:
     def get_function_page(
         self, path: str, filename: str
     ) -> Tuple[FunctionOrNone, str, bool]:
-
         """
         This function find function from URL path.
         If the function is a WebScripts built-in function,
@@ -776,7 +749,6 @@ class Server:
 
     @log_trace
     def get_URLs(self) -> List[str]:
-
         """
         This function return a list of urls (scripts, documentation...)
         and the start of the URL of custom packages.
@@ -814,7 +786,6 @@ class Server:
         attributes: List[str],
         is_not_package: bool = True,
     ) -> Tuple[FunctionOrNone, bool]:
-
         """
         This function get recursive attribute from object.
         """
@@ -864,7 +835,6 @@ class Server:
     def get_inputs(
         arguments: List[Dict[str, JsonValue]]
     ) -> Tuple[List[str], List[str]]:
-
         """
         This function returns inputs and arguments from arguments.
         """
@@ -876,13 +846,11 @@ class Server:
         logger_debug("Extract inputs from WebScripts arguments...")
         for i, argument in enumerate(arguments):
             if argument["input"]:
-                # logger_debug(f"Input found: {argument}")
                 # To protect secrets, do not log arguments !
                 append(argument)
 
         logger_debug("Remove inputs from arguments...")
         for i, input_ in enumerate(inputs):
-            # logger_debug(f"Remove {input_}")
             # To protect secrets, do not log arguments !
             remove(input_)
             inputs[i] = str(input_["value"])
@@ -896,7 +864,6 @@ class Server:
     @staticmethod
     @log_trace
     def get_content_length(environ: _Environ) -> int:
-
         """
         This function returns the content length.
         """
@@ -915,7 +882,6 @@ class Server:
     def try_get_command(
         body: Dict[str, JsonValue]
     ) -> Union[None, Tuple[Content, str, bool]]:
-
         """
         This function returns arguments, CSRF token and True if is WebScripts
         request. If is not a WebScripts request because there's no "arguments"
@@ -944,7 +910,6 @@ class Server:
     @staticmethod
     @log_trace
     def get_baseurl(environ_getter: Callable, environ: _Environ) -> str:
-
         """
         This function returns URL base.
         """
@@ -963,7 +928,6 @@ class Server:
     @staticmethod
     @log_trace
     def get_fullurl(environ: _Environ) -> str:
-
         """
         This function returns the full URL (based on the PEP 3333).
 
@@ -999,7 +963,6 @@ class Server:
     @staticmethod
     @log_trace
     def check_origin(environ_getter: Callable, environ: _Environ) -> bool:
-
         """
         This function checks Origin of POST methods.
         """
@@ -1018,7 +981,6 @@ class Server:
     @staticmethod
     @log_trace
     def get_json_content(body: bytes, content_type: str) -> JsonValue:
-
         """
         This functions returns the loaded JSON content.
         """
@@ -1038,7 +1000,6 @@ class Server:
 
     @log_trace
     def parse_body(self, environ: _Environ) -> Tuple[Content, str, bool]:
-
         """
         This function returns arguments from body.
         """
@@ -1080,24 +1041,26 @@ class Server:
 
     @log_trace
     def app(self, environ_: _Environ, respond: MethodType) -> List[bytes]:
-
         """
         This function get function page,
         return content page, catch errors and
         return HTTP errors.
         """
 
-        environ = self.environ._data.copy()
+        # environ: _Environ = self.environ
+        # encodevalue: Callable = environ.encodevalue
+        # encodekey: Callable = environ.encodekey
+        environ = {key: value for key, value in self.environ.items()}
         # environ = _Environ(
-        #     environ._data,
-        #     environ.encodekey,
+        #     environ._data.copy(),
+        #     encodekey,
         #     environ.decodekey,
-        #     environ.encodevalue,
+        #     encodevalue,
         #     environ.decodevalue
         # )
 
         # d_setitem: Callable = environ._data.__setitem__
-        # [d_setitem(encode_key(key), encode_value(value)) if isinstance(value, str) else d_setitem(encode_key(key), value) for key, value in environ_.items()]
+        # [d_setitem(encodekey(key), encodevalue(value)) if isinstance(value, str) else d_setitem(encodekey(key), value) for key, value in environ_.items()]
 
         environ.update(environ_)
 
@@ -1247,7 +1210,6 @@ class Server:
     def set_default_values_for_response(
         self, error: str, headers: Dict[str, str]
     ) -> Tuple[str, Dict[str, str]]:
-
         """
         This function returns default error if not defined and
         default headers updated with custom headers.
@@ -1266,7 +1228,6 @@ class Server:
     @staticmethod
     @log_trace
     def return_page(page: Union[bytes, str, Iterable[bytes]]) -> List[bytes]:
-
         """
         This function returns response as a list of bytes.
         """
@@ -1287,7 +1248,6 @@ class Server:
         arguments: List[Dict[str, JsonValue]],
         is_webscripts_request: bool,
     ) -> Tuple[List[str], List[str]]:
-
         """
         This function returns inputs (using Server.get_inputs).
         """
@@ -1312,7 +1272,6 @@ class Server:
         error: str = None,
         headers: Dict[str, str] = None,
     ) -> None:
-
         """
         This function send error code, message and headers.
         """
@@ -1343,7 +1302,6 @@ class Server:
         error: Union[str, bytes, Iterable[bytes]],
         respond: MethodType,
     ) -> List[bytes]:
-
         """
         This function return error 500 web page.
         """
@@ -1368,7 +1326,6 @@ class Server:
         url: str,
         respond: MethodType,
     ):
-
         """
         This function return error 404 web page.
         """
@@ -1395,7 +1352,6 @@ class Server:
         method: str,
         respond: MethodType,
     ):
-
         """
         This function return error 400 web page.
         """
@@ -1419,7 +1375,6 @@ class Server:
         error_description: str,
         respond: MethodType,
     ):
-
         """
         This function return error 401 web page.
         """
@@ -1440,7 +1395,6 @@ class Server:
         error_description: str,
         respond: MethodType,
     ):
-
         """
         This function return error 403 web page.
         """
@@ -1461,7 +1415,6 @@ class Server:
         error_description: str,
         respond: MethodType,
     ):
-
         """
         This function return error 406 web page.
         """
@@ -1485,7 +1438,6 @@ class Server:
         data: bytes,
         respond: MethodType,
     ) -> List[bytes]:
-
         """
         This function send HTTP errors.
         """
@@ -1540,7 +1492,6 @@ class Server:
         error: str,
         code: str,
     ) -> Tuple[str, Dict[str, str], str]:
-
         """
         This function call custom errors pages.
         """
@@ -1556,7 +1507,6 @@ class Server:
             return function(environ, user, self, filename, error)
 
         packages = self.pages.packages
-        # for package in self.pages.packages.__dict__.values():
         for package in dir(packages):
             package = getattr(packages, package)
 
@@ -1580,7 +1530,6 @@ class Server:
 
 @log_trace
 def parse_args(argv: List[str] = argv) -> Namespace:
-
     """
     This function parse command line arguments.
     """
@@ -1808,8 +1757,9 @@ def parse_args(argv: List[str] = argv) -> Namespace:
 
 
 @log_trace
-def get_server_config(arguments: Namespace) -> Iterator[dict]:
-
+def get_server_config(
+    arguments: Namespace, secure: bool = False
+) -> Iterator[dict]:
     """
     This generator return configurations dict.
     """
@@ -1822,7 +1772,7 @@ def get_server_config(arguments: Namespace) -> Iterator[dict]:
     insert = paths.insert
 
     temp_config = Configuration()
-    temp_config.force_file_permissions = True
+    temp_config.force_file_permissions = secure
 
     if system() == "Windows":
         logger_debug("Add default server configuration for Windows...")
@@ -1882,7 +1832,6 @@ def get_server_config(arguments: Namespace) -> Iterator[dict]:
 
 @log_trace
 def logs_configuration(configuration: NameSpace) -> None:
-
     """
     This function configure ROOT logger from
     configuration files and command line arguments.
@@ -1921,7 +1870,6 @@ def logs_configuration(configuration: NameSpace) -> None:
 def add_configuration(
     configuration: Configuration, config: Dict[str, JsonValue]
 ) -> Configuration:
-
     """
     This function add configuration in ServerConfiguration.
     """
@@ -1954,8 +1902,7 @@ def add_configuration(
     return configuration
 
 
-def configure_logs_system() -> Tuple[Set[str], Set[str]]:
-
+def configure_logs_system(secure: bool = False) -> Tuple[Set[str], Set[str]]:
     """
     This function try to create the logs directory
     if not found and configure logs.
@@ -1976,7 +1923,7 @@ def configure_logs_system() -> Tuple[Set[str], Set[str]]:
     log_file = get_real_path(join("config", "loggers.ini"))
 
     temp_config = Configuration()
-    temp_config.force_file_permissions = True
+    temp_config.force_file_permissions = secure
 
     if not check_file_permission(temp_config, log_file):
         raise WebScriptsSecurityError(
@@ -1991,18 +1938,6 @@ def configure_logs_system() -> Tuple[Set[str], Set[str]]:
 
     Logs.log_response.handlers[0].baseFilename
 
-    # basicConfig(
-    #     format=(
-    #         "%(asctime)s %(levelname)s %(message)s (%(funcName)s -> "
-    #         "%(filename)s:%(lineno)d)"
-    #     ),
-    #     datefmt="%d/%m/%Y %H:%M:%S",
-    #     encoding="utf-8",
-    #     level=0,
-    #     filename=join(get_real_path("logs", is_dir=True), "root.logs"),
-    #     force=True,
-    # )
-
     logs_path = set()
     log_files = set()
 
@@ -2038,14 +1973,8 @@ def configure_logs_system() -> Tuple[Set[str], Set[str]]:
     environ["WEBSCRIPTS_LOGS_FILES"] = "|".join(log_files)
     return logs_path, log_files
 
-    #     if logger.hasHandlers() and len(logger.handlers):
-    #         logger.handlers[0].doRollover = Handler.doRollover
-    #         logger.handlers[0].rotator = Handler.rotator
-    #         logger.handlers[0].namer = Handler.namer
-
 
 def send_mail(*args, **kwargs) -> int:
-
     """
     This function send a mail to adminitrators
     using the error_pages modules.
@@ -2068,13 +1997,14 @@ def send_mail(*args, **kwargs) -> int:
     return 1
 
 
-def default_configuration(argv: List[str] = argv) -> Configuration:
-
+def default_configuration(
+    argv: List[str] = argv, secure: bool = False
+) -> Configuration:
     """
     This function builds the default configuration.
     """
 
-    log_paths, log_files = configure_logs_system()
+    log_paths, log_files = configure_logs_system(secure)
     environ["WEBSCRIPTS_PATH"] = server_path
     args = parse_args(argv)
 
@@ -2083,11 +2013,10 @@ def default_configuration(argv: List[str] = argv) -> Configuration:
     configuration = Configuration()
     configuration.logs_path = list(log_paths)
     configuration.log_files = list(log_files)
-    for config in get_server_config(args):
+    for config in get_server_config(args, secure):
         configuration = add_configuration(configuration, config)
 
     configuration = add_configuration(configuration, args.__dict__)
-    # logs_configuration(configuration)
 
     logger_debug("Check and type configurations...")
     configuration.set_defaults()
@@ -2121,6 +2050,7 @@ def default_configuration(argv: List[str] = argv) -> Configuration:
     configuration.data_dir = datapath = get_real_path(
         getattr(configuration, "data_dir", "data"), is_dir=True
     )
+
     environ["WEBSCRIPTS_DATA_PATH"] = datapath
     environ["WEBSCRIPTS_DOCUMENTATION_PATH"] = ":".join(
         configuration.documentations_path
@@ -2131,8 +2061,25 @@ def default_configuration(argv: List[str] = argv) -> Configuration:
     return configuration
 
 
-def main() -> int:
+def prepare_server(secure: bool = True) -> Server:
+    """
+    This function prepares server to be launched securly.
+    """
 
+    configuration = default_configuration(argv, secure)
+    debug = getattr(configuration, "debug", None)
+
+    if debug:
+        logger_debug("Debug mode detected: export configuration...")
+        configuration.export_as_json()
+
+    logger_debug("Build server with configurations...")
+    server = Server(configuration)
+
+    return server, debug
+
+
+def main() -> int:
     """
     Main function to build the
     configurations and launch the server.
@@ -2144,18 +2091,9 @@ def main() -> int:
     else:
         NO_START = False
 
-    configuration = default_configuration(argv)
-    debug = getattr(configuration, "debug", None)
-
-    if debug:
-        # first export to get server configuration on
-        # Script Configuration Error
-        logger_debug("Debug mode detected: export configuration...")
-        configuration.export_as_json()
-
-    logger_debug("Build server with configurations...")
-    server = Server(configuration)
+    secure: bool = "--security" not in argv
 
+    server, debug = prepare_server(secure)
     httpd = simple_server.make_server(
         server.interface, server.port, server.app
     )
@@ -2163,17 +2101,18 @@ def main() -> int:
 
     logger_debug("Trying to send email notification...")
     send_mail(
-        configuration,
+        server.configuration,
         f"Server is up on http://{server.interface}:{server.port}/.",
     )
 
     logger_info("Check hardening of the WebScripts server...")
-    hardening(server)
+    if secure:
+        hardening(server)
 
     if debug:
         # second export to get all configurations
         logger_debug("Debug mode detected: export configuration...")
-        configuration.export_as_json()
+        server.configuration.export_as_json()
 
     logger_warning(
         f"Starting server on http://{server.interface}:{server.port}/ ..."
@@ -2195,7 +2134,7 @@ def main() -> int:
 
     logger_debug("Trying to send email notification...")
     send_mail(
-        configuration,
+        server.configuration,
         f"Server is down on http://{server.interface}:{server.port}/.",
     )
     logger_debug("Exit with code 0.")
