@@ -3,7 +3,7 @@
 
 ###################
 #    This file can decrypt and print a secure password share
-#    Copyright (C) 2021  Maurice Lambert
+#    Copyright (C) 2021, 2022, 2023  Maurice Lambert
 
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 ###################
 
-"""This tool run scripts and display the result in a Web Interface.
+"""#    This tool runs CLI scripts and displays output in a Web Interface.
 
 This file can decrypt and print a secure password share."""
 
@@ -29,14 +29,14 @@ __author_email__ = "mauricelambert434@gmail.com"
 __maintainer__ = "Maurice Lambert"
 __maintainer_email__ = "mauricelambert434@gmail.com"
 __description__ = """
-This tool run scripts and display the result in a Web Interface.
+This tool runs CLI scripts and displays output in a Web Interface.
 
 This file can decrypt and print a secure password share."""
 __license__ = "GPL-3.0 License"
 __url__ = "https://github.com/mauricelambert/WebScripts"
 
 copyright = """
-WebScripts  Copyright (C) 2021  Maurice Lambert
+WebScripts  Copyright (C) 2021, 2022, 2023  Maurice Lambert
 This program comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions.
@@ -65,7 +65,6 @@ filename = path.join("data", "passwords.csv")
 def decrypt(
     key: bytes, password: bytes, hash_: str, iteration: int
 ) -> PasswordOrFalse:
-
     """This function checks the integrity of the
     password and returns the decrypted password."""
 
@@ -86,7 +85,6 @@ def decrypt(
 def get_passwords(
     id_: int,
 ) -> Tuple[List[List[PasswordInfo]], List[PasswordInfo]]:
-
     """This function returns a list of passwords
     and the requested password."""
 
@@ -109,7 +107,6 @@ def get_passwords(
 
 
 def main() -> None:
-
     """Main function to obtain secure password sharing:
     - Check the validity of the token
     - Check the validity of the password
@@ -173,7 +170,6 @@ def main() -> None:
 
 
 def delete(passwords: List[List[PasswordInfo]]) -> List[List[PasswordInfo]]:
-
     """This function delete old passwords."""
 
     for password in passwords:
@@ -183,7 +179,6 @@ def delete(passwords: List[List[PasswordInfo]]) -> List[List[PasswordInfo]]:
 
 
 def save(passwords: List[List[PasswordInfo]]) -> None:
-
     """This function re-saves passwords."""
 
     with open(filename, "w", newline="") as file:
